@echo off

call "%USERPROFILE%\.mosamatic\MosamaticDesktop\Scripts\activate"
call "%USERPROFILE%\.mosamatic\MosamaticDesktop\Scripts\mosamatic-desktop"
call "%USERPROFILE%\.mosamatic\MosamaticDesktop\Scripts\deactivate"
